use Test::More tests => 14;

# Test case without init() and cleanup()
package My::Test1;
use base 'DE_EPAGES::Core::API::Test';
use Test::More;
use strict;

sub test {
    my $self = shift;
    pass 'test() called once';
    isa_ok( $self, __PACKAGE__, 'self passed to test' );
}

sub run {
    my $self = shift;

    pass 'run() called once by runScript';
    $self->SUPER::run;
}

__PACKAGE__->runScript;

# Test case with init() and cleanup()
package My::InitCleanup; ## no critic qw(ProhibitMultiplePackages)
use base 'DE_EPAGES::Core::API::Test';
use Test::More;
use strict;

sub new {
    my $class = shift;
    my $self = $class->SUPER::new;

    $self->{'init'} = 0;
    $self->{'cleanup'} = 0;
    $self->{'test'} = 0;
    return $self;
}

sub test {
    my $self = shift;

    is( $self->{'cleanup'}, 1, 'cleanup() was called before test()' );
    is( $self->{'init'}, 1, 'init() was called before test()' );
    is( $self->{'test'}, 0, 'test() is called only once' );
    $self->{'test'}++;
}

sub init {
    my $self = shift;

    is( $self->{'init'}, 0, 'init() is called only once' );
    $self->{'init'}++;
    is( $self->{'cleanup'}, 1, 'cleanup() was called before init()' );
}

sub cleanup {
    my $self = shift;
    pass 'cleanup() called';
    $self->{'cleanup'}++;
}

__PACKAGE__->new->run;

# Test case with error and cleanup()
package My::TestWithError; ## no critic qw(ProhibitMultiplePackages)
use base 'DE_EPAGES::Core::API::Test';
use Test::More;

sub new {
    my $class = shift;
    my $self = $class->SUPER::new;

    $self->{'cleanup'} = 0;
    return $self;
}

sub test {
    my $self = shift;

    die 'error in test';
}

sub cleanup {
    my $self = shift;
    pass 'cleanup() called';
    $self->{'cleanup'}++;
}

my $Test = __PACKAGE__->new;
eval { $Test->run; };
ok( $@, 'error occured' );
is( $Test->{'cleanup'}, 2, 'cleanup() was called twice despite of the error' );
