#========================================================================================
# package      MessageBuilderTest
# state        private
# description  Message Build test
#========================================================================================
package MessageBuilderTest;
use strict;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::DocdataPay::API::SOAP::MessageBuilder;
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use Test::More;
use Test::Exception;
use Readonly;
use Data::Dumper;

Readonly my $ROOTPATH => '/Test';

## no critic ( ProhibitNestedSubs ProhibitPathConcatenation ProhibitMultiplePackages ProhibitPrivateSubroutineUse ProhibitMagicNumbers)
sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

            # tests
            test_gross2net();
            test_getCreateMessageData();
            test_buildCreateMessage();
            test_buildStatusMessage();
            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub test_gross2net {
    note((caller(0))[3]);
    my $Builder = DE_EPAGES::DocdataPay::API::SOAP::MessageBuilder->new();
    $Builder->{OrderOrBasket} = LoadObjectByPath($ROOTPATH.'/TestShop');
    $Builder->{hData}{hLineItemContainerData}{CurrencyID} = 'EUR';

    my $GrossValue = 100;
    my $TaxClass = LoadObjectByPath('/TaxMatrixGermany/normal');
    my $Result = $Builder->_gross2net($GrossValue, $TaxClass);
    my $Expected = 84.03;
    is($Result, $Expected, "test_gross2net got $Result and as expected $Expected");
}
sub test_getCreateMessageData {
    note((caller(0))[3]);
    my $Builder = DE_EPAGES::DocdataPay::API::SOAP::MessageBuilder->new();
    $Builder->{OrderOrBasket} = LoadObjectByPath($ROOTPATH.'/TestShop/Customers/1001/Orders/1005');
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = GetPKeyLanguageByCode( 'en' );

    lives_ok { $Builder->_getCreateMessageData(); } '_getCreateData ran without error';
}

sub test_buildCreateMessage {
    note((caller(0))[3]);
    my $Builder = DE_EPAGES::DocdataPay::API::SOAP::MessageBuilder->new();
    $Builder->{OrderOrBasket} = LoadObjectByPath($ROOTPATH.'/TestShop/Customers/1001/Orders/1005');
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = GetPKeyLanguageByCode( 'en' );

    my $Result;
    lives_ok {$Result = $Builder->_buildCreateMessage(); } '_buildCreateMessage ran without error';
}

sub test_buildStatusMessage {
    note((caller(0))[3]);
    my $Builder = DE_EPAGES::DocdataPay::API::SOAP::MessageBuilder->new();
    $Builder->{OrderOrBasket} = LoadObjectByPath($ROOTPATH.'/TestShop/Customers/1001/Orders/1005');
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = GetPKeyLanguageByCode( 'en' );

    my $Result;
    lives_ok {$Result = $Builder->_buildStatusMessage(); } '_buildStatusMessage ran without error';
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects( 'ImportOrder.xml', $ROOTPATH );
    return 1;
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

RunScript( Sub => \&Main );

## use critic ( ProhibitNestedSubs ProhibitPathConcatenation ProhibitMultiplePackages ProhibitPrivateSubroutineUse ProhibitMagicNumbers)
1;