/*global define, window, contentslider*/

define([
    'testsuite/qunit',
    'jquery',
    './contentslider-view',

    '$ready!'
], function (QUnit, $, ContentsliderView) {
    'use strict';

    var fileManager = [{
            "currentGallerySize": "1344",
            "children": [{
                "files": [{
                    "width": "900",
                    "extension": ".png",
                    "name": "contentslider-testimage1.png",
                    "path": "contentslider-testimage1.png",
                    "height": "300",
                    "directory": "/MediaGallery/",
                    "modified": 1200994146,
                    "size": 67089,
                    "fullpath": "../ui/contentslider-testimage1.png",
                    "selectable": 1,
                    "mimetype": "image/png"
                }, {
                    "width": "1050",
                    "extension": ".png",
                    "name": "contentslider-testimage2.png",
                    "path": "/contentslider-testimage2.png",
                    "height": "350",
                    "directory": "/MediaGallery",
                    "modified": 1200994146,
                    "size": 67089,
                    "fullpath": "../ui/contentslider-testimage2.png",
                    "selectable": 1,
                    "mimetype": "image/png"
                }],
                "children": [],
                "metadata": {
                    "displayname": "MediaGallery",
                    "type": "MediaGallery",
                    "path": ""
                },
                "data": "MediaGallery",
                "state": "closed"
            }],
            "maxGallerySize": "102400"
        }],

        _ajax = $.ajax;

    $.ajax = function (o) {
        var result = $.Deferred(),
            jqXHR = result.promise();

        if (o.data) {
            if (o.data.ViewAction === "JSONInitMediaGallery") {
                result.resolve(fileManager);
            }
        } else {
            return _ajax(o);
        }

        return result;
    };

    QUnit.module('de_epages/externalcontent/ui/contentslider-view', {
        setup: function () {
            window.contentslider = new ContentsliderView();
            $('#qunit-fixture').append(contentslider.render().el);
        },
        teardown: function () {
            $('#qunit-fixture').empty();
            $('.ui-dialog').remove();
        }
    });

    QUnit.test('Test content slider view methods', 13, function () {
        $.each(['onColorSelect', 'onSlide', 'onSortstart', 'onSortupdate', 'deleteSlide', 'linkSlide', 'editSlide', 'renderPreview', 'showLinkInfo', 'render', 'initialize', 'addImage', 'cropImage'], function (i, method) {
            QUnit.equal(
                $.isFunction(contentslider[method]),
                true,
                method + ' is a method of contentslider-view'
            );
        });
    });

    QUnit.test('Test content slider settings', function () {
        QUnit.equal(
            $('#qunit-fixture').children().length,
            1,
            'Create content slider view'
        );

        QUnit.deepEqual(
            contentslider.gOptions,
            {"effect": "fade", "arrows": false, "shadow": false, "border": "transparent", "automatic": true, "displaytime": 3, "slides": []},
            'Get default options'
        );

        // set border color
        contentslider.elements.borderColorpicker.trigger('click');
        $('.ep-uiColorpicker-hexInput').val('#000000').trigger('change');
        $('.ep-uiColorpicker-applyButton').trigger('click');

        QUnit.equal(
            contentslider.gOptions.border,
            'black',
            'Change border color to *black*'
        );

        // change some default options
        $('#arrows').trigger('click');
        $('#shadow').trigger('click');
        $('#automatic').trigger('click');

        QUnit.deepEqual(
            contentslider.gOptions,
            {"effect": "fade", "arrows": true, "shadow": true, "border": "black", "automatic": false, "displaytime": 3, "slides": []},
            'Change options'
        );

        // open filemanager und choose an image
        contentslider.elements.addImage.trigger('click');
        $('.ep-mediagallery-fileContainer:first').trigger('click');
        $('.ui-dialog-buttonset button').trigger('click');

        contentslider.addImage('contentslider-testimage1.png', '../ui/contentslider-testimage1.png', undefined);

        QUnit.equal(
            $('.ep-fileName').text(),
            'contentslider-testimage1.png',
            'Add *contentslider-testimage1.png*'
        );

        $('.ep-sprite.ico_s_edit').trigger('click');
        contentslider.elements.addImage.mediagalleryUiFilemanagerdialog('close');
        contentslider.addImage('contentslider-testimage2.png', '../ui/contentslider-testimage2.png', 0);

        QUnit.equal(
            $('.ep-fileName').text(),
            'contentslider-testimage2.png',
            'Replace *contentslider-testimage1.png* by *contentslider-testimage1.png*'
        );

        $('.ep-sprite.ico_s_delete').trigger('click');
        $('.ui-dialog-buttonset button').trigger('click');

        QUnit.equal(
            $('.ep-fileName').length,
            0,
            'Remove slide *contentslider-testimage2.png*'
        );

        contentslider.addImage('contentslider-testimage1.png', '../ui/contentslider-testimage1.png', undefined);

        $('.ep-sprite.object_ico_s_link').trigger('click');
        $('#ep-linkpicker-externalUrl').val('www.epages.com').trigger('keyup');
        $('.linkpickerDialog').siblings('.ui-dialog-buttonpane').find('button:last').trigger('click');

        QUnit.equal(
            contentslider.gOptions.slides[0].href,
            'http://www.epages.com',
            'Add link to slide'
        );

    });
});