/*global define, window, maps, ep*/

define([
    'testsuite/qunit',
    'jquery',
    './maps-view',

    '$ready!'
], function (QUnit, $, MapsView) {
    'use strict';

    // Google Maps API for testing
    window.google = {
        load: function (type, version, params) {
            params.callback();
        },
        maps: {
            LatLng: function () {
                return {};
            },
            Marker: function () {
                return {
                    setTitle: function () {},
                    setMap: function () {},
                    setDraggable: function () {}
                };
            },
            Map: function () {
                return {};
            },
            InfoWindow: function () {
                return {};
            },
            event: {
                addListener: function () {
                    return {};
                }
            }
        }
    };

    // Bing API for testing
    window.initBingMapIsExecuted = false;
    window.VEMapStyle = [];
    window.VEMapMode = [];
    window.VEMap = function (id) {
        this.id = id;
        this.LoadMap = function (location, zoom, type, fixedView, mode) {
            window.initBingMapIsExecuted = true;
            return {};
        };
        this.HideDashboard = function () {
            return {};
        };
        this.AddPushpin = function (location) {
            return { SetDescription: function (title) {} };
        };
        this.Dispose = function () {
            return {};
        };
        this.ShowInfoBox = function () {
            return {};
        };
        this.AttachEvent = function () {
            return {};
        };
    };

    window.VELatLong = function (lat, lng) {
        return {};
    };

    $.ajax = function (o) {
        return {done: function (_func) { _func(); } };
    };

    ep.config.shopAddress = 'Leutragraben 1, 07743 Jena';

    QUnit.module('de_epages/externalcontent/ui/maps-view', {
        setup: function () {
            window.maps = new MapsView();
            $('#qunit-fixture').append(maps.render().el);
        },
        teardown: function () {
            $('#qunit-fixture').empty();
        }
    });

    QUnit.test('Test maps view methods', 22, function () {
        // test general methods of maps view
        $.each(['onApply', 'onRadio', 'addressToLatLng', 'renderPreview', 'render'], function (i, method) {
            QUnit.equal(
                $.isFunction(maps[method]),
                true,
                method + ' is a method of maps-view'
            );
        });

        // test google maps settings
        QUnit.equal(
            $('fieldset[name="type"] input[value="google"]:checked').length,
            1,
            'Google maps is selected'
        );

        QUnit.equal(
            $('fieldset[name="bing"]').attr('disabled'),
            'disabled',
            'Bing maps settings are disabled'
        );

        QUnit.equal(
            $('fieldset[name="klicktel"]').attr('disabled'),
            'disabled',
            'Klicktel maps settings are disabled'
        );

        QUnit.deepEqual(
            maps.gOptions,
            {"type": "google", "width": 500, "height": 300, "zoom": 15, "title": "", "controls": true, "fixedView": false},
            "Get default options"
        );

        $('fieldset[name="google"] #address').val('Jena').trigger('change');
        $('fieldset[name="google"] #title').val('My title').trigger('change');
        $('fieldset[name="google"] #height').val('400').trigger('change');
        $('fieldset[name="google"] #width').val('400').trigger('change');
        $('fieldset[name="google"] #controls').trigger('click');
        $('fieldset[name="google"] #fixedView').trigger('click');

        QUnit.deepEqual(
            maps.gOptions,
            {"type": "google", "width": "400", "height": "400", "zoom": 15, "title": "My title", "controls": false, "fixedView": true, "address": "Jena"},
            "Get changed options"
        );

        // test bing maps settings
        $('fieldset[name="type"] input[value="bing"]').trigger('click');

        QUnit.equal(
            $('fieldset[name="type"] input[value="bing"]:checked').length,
            1,
            'Bing maps is selected'
        );

        QUnit.equal(
            $('fieldset[name="google"]').attr('disabled'),
            'disabled',
            'Google maps settings are disabled'
        );

        QUnit.equal(
            $('fieldset[name="klicktel"]').attr('disabled'),
            'disabled',
            'Klicktel maps settings are disabled'
        );

        QUnit.deepEqual(
            maps.gOptions,
            {"type": "bing", "height": "300", "width": "500", "controls": true, "fixedView": false},
            "Get default options"
        );

        $('fieldset[name="bing"] #address').val('Jena').trigger('change');
        $('fieldset[name="bing"] #title').val('My title').trigger('change');
        $('fieldset[name="bing"] #height').val('400').trigger('change');
        $('fieldset[name="bing"] #width').val('400').trigger('change');
        $('fieldset[name="bing"] #controls').trigger('click');
        $('fieldset[name="bing"] #fixedView').trigger('click');

        QUnit.deepEqual(
            maps.gOptions,
            {"type": "bing", "height": "400", "width": "400", "controls": false, "fixedView": true, "address": "Jena", "title": "My title"},
            "Get changed options"
        );

        // test klicktel maps settings
        $('fieldset[name="type"] input[value="klicktel"]').trigger('click');

        QUnit.equal(
            $('fieldset[name="type"] input[value="klicktel"]:checked').length,
            1,
            'Klicktel maps is selected'
        );

        QUnit.equal(
            $('fieldset[name="google"]').attr('disabled'),
            'disabled',
            'Google maps settings are disabled'
        );

        QUnit.equal(
            $('fieldset[name="bing"]').attr('disabled'),
            'disabled',
            'Klicktel maps settings are disabled'
        );

        QUnit.deepEqual(
            maps.gOptions,
            {"type": "klicktel", "address": "", "contactPerson": "", "email": "", "width": "400", "height": "500", "useRoute": false, "showAddress": false, "largeNavigation": false, "layoutLR": false},
            "Get default options"
        );

        $('fieldset[name="klicktel"] #address').val('Jena').trigger('change');
        $('fieldset[name="klicktel"] #contactPerson').val('Max Mustermann').trigger('change');
        $('fieldset[name="klicktel"] #email').val('m.mustermann@epages.de').trigger('change');
        $('fieldset[name="klicktel"] #height').val('400').trigger('change');
        $('fieldset[name="klicktel"] #width').val('400').trigger('change');
        $('fieldset[name="klicktel"] #useRoute').trigger('click');
        $('fieldset[name="klicktel"] #showAddress').trigger('click');
        $('fieldset[name="klicktel"] #largeNavigation').trigger('click');
        $('fieldset[name="klicktel"] #layoutLR').trigger('click');

        QUnit.deepEqual(
            maps.gOptions,
            {"type": "klicktel", "address": "Jena", "contactPerson": "Max Mustermann", "email": "m.mustermann@epages.de", "width": "400", "height": "400", "useRoute": true, "showAddress": true, "largeNavigation": true, "layoutLR": true},
            "Get changed options"
        );

        QUnit.equal(
            maps.onApply(),
            true,
            "Validate form with valid data"
        );

        $('fieldset[name="klicktel"] #address').val('').trigger('change');

        QUnit.equal(
            maps.onApply(),
            false,
            "Validate form after required value was removed"
        );
    });
});