/*global define*/

define([
    'testsuite/qunit',
    'jquery',

    './contentslider',
    '$ready!'
], function (QUnit, $) {
    QUnit.module('de_epages/externalcontent/ui/contentslider');

    QUnit.test('Test contentslider methods', 4, function () {
        $.each(['_create', '_setOption', '_createCycle', 'destroy'], function (i, fn) {
            QUnit.equal(
                $.isFunction($.ui.externalcontentUiContentslider.prototype[fn]),
                true,
                fn + 'is a method of contentslider'
            );
        });
    });

    QUnit.asyncTest('Test contentslider', 14, function () {
        // Room for our test cases.
        var slider = [],
            instances = [],
            plugin = 'externalcontentUiContentslider';

        // Create *contentslider*.
        slider[0] = $('#contentslider0')[plugin]({slides: []});

        slider[1] = $('#contentslider1')[plugin]({
            automatic: false,
            arrows: false,
            effect: 'fade',
            shadow: true,
            border: 'transparent',
            displaytime: 3,
            slides: [{
                'fullpath': 'contentslider-testimage1.png',
                'name': 'Test1'
            }]
        });

        // For testing the *cycleOptions* parameter.
        window.calledBefore = 0;
        slider[2] = $('#contentslider2')[plugin]({
            automatic: true,
            arrows: true,
            effect: 'scrollLeft',
            shadow: false,
            border: 'black',
            automatic: false,
            displaytime: 5,
            cycleOptions: {
                before: function () {
                    window.calledBefore = window.calledBefore + 1;
                }
            },
            slides: [{
                'fullpath': 'contentslider-testimage1.png',
                'name': 'Test1',
                'href': 'http://somelink.de',
                'title': 'Some Link'
            }, {
                'fullpath': 'contentslider-testimage1.png',
                'name': 'Test2'
            }]
        });

        QUnit.strictEqual(
            window.calledBefore,
            0,
            '*window.calledBefore* is 0, because *before* callback has not been called yet'
        );

        QUnit.strictEqual(
            slider[0].children().length,
            0,
            'No *slides* => no *contentslider*. *slider[0].children().length*'
        );

        QUnit.strictEqual(
            slider[1].hasClass('contentslider-shadow'),
            true,
            'Option *shadow* is true. *slider[1].hasClass(\'contentslider-shadow\')*'
        );

        // map transparent for Firefox, IE
        var borderColor = slider[1].css('border-top-color')
        if (borderColor === 'transparent') {
            borderColor = 'rgba(0, 0, 0, 0)';
        }

        QUnit.strictEqual(
            borderColor,
            'rgba(0, 0, 0, 0)',
            'Option *border* is "transparent". *slider[1].css(\'border-color\')*'
        );

        QUnit.strictEqual(
            slider[1].children().length,
            2,
            '*slider[1]* has *pager* `div` and *list* `ul`. *slider[1].children().length*'
        );

        QUnit.strictEqual(
            slider[2].hasClass('contentslider-shadow'),
            false,
            'Option *shadow* is false. *slider[2].hasClass(\'contentslider-shadow\')*'
        );

        QUnit.strictEqual(
            slider[2].css('border-top-color'),
            'rgb(0, 0, 0)',
            'Option *border* is "black". *slider[2].css(\'border-color\')*'
        );

        QUnit.strictEqual(
            slider[2].children().length,
            3,
            '*slider[2]* has *pager* `div`, *list* `ul` and *arrows* `div`. *slider[1].children().length*'
        );

        setTimeout(function () {

            QUnit.strictEqual(
                window.calledBefore,
                1,
                '*window.calledBefore* is 1, because *before* callback was called once after initialization.'
            );

            QUnit.strictEqual(
                slider[0].width(),
                $('#qunit-fixture').width(),
                '*slider[0]* `div` has width of surrounding `div`'
            );

            QUnit.strictEqual(
                slider[1].width(),
                $('#contentslider1-wrapper').width(),
                '*slider[1]* `div` has width of wrapper `div`'
            );

            QUnit.strictEqual(
                slider[2].width(),
                $('#qunit-fixture').width(),
                '*slider[2]* `div` has width of parent `div`'
            );

            slider[1][plugin]('destroy');

            QUnit.strictEqual(
                slider[1].data('externalcontentUiContentslider'),
                void(0),
                '*slider[1]* was *destroy*ed'
            );

            slider[2][plugin]('destroy');

            QUnit.strictEqual(
                slider[2].data('externalcontentUiContentslider'),
                void(0),
                '*slider[2]* was *destroy*ed'
            );

            QUnit.start();

        }, 500);
    });

});