/*global define*/

define([
    'testsuite/qunit',
    'jquery',

    './cropslidedialog',
    '$ready!'
], function (QUnit, $) {
    QUnit.module('de_epages/externalcontent/ui/cropslidedialog');

    QUnit.test('Test cropslidedialog methods', 6, function () {
        $.each(['_create', '_setOption', '_onImageLoad', 'getFabricCanvas', 'getFabricImage', 'destroy'], function (i, fn) {
            QUnit.equal(
                $.isFunction($.ui.externalcontentUiCropslidedialog.prototype[fn]),
                true,
                fn + ' is a method of cropslidedialog'
            );
        });
    });

    QUnit.asyncTest('Test cropslidedialog', 29, function () {
        // References for later testing.
        var image, canvas, widget = 'externalcontentUiCropslidedialog';

        // 1st case.
        // Width Too Small.
        var test = $('<div></div>').externalcontentUiCropslidedialog({
            width: 1000,
            height: 200,
            url: 'cropslidedialog-testimage.png'
        });

        // Wait for image to load.
        setTimeout(function () {

            QUnit.strictEqual(
                test.externalcontentUiCropslidedialog('option', 'width'),
                1000,
                "test.externalcontentUiCropslidedialog('option', 'width')"
            );

            QUnit.strictEqual(
                test.externalcontentUiCropslidedialog('option', 'height'),
                200,
                "test.externalcontentUiCropslidedialog('option', 'height')"
            );

            canvas = test[widget]('getFabricCanvas');
            image = test[widget]('getFabricImage');

            QUnit.strictEqual(canvas.width, 1000, 'Width of inserted canvas. canvas.width');
            QUnit.strictEqual(canvas.height, 200, 'Height of inserted canvas. canvas.height');

            QUnit.strictEqual(canvas.item(0) === image, true, 'Image inserted into canvas. canvas.item(0) === image');
            QUnit.strictEqual(image.lockMovementX, true, 'Image height too big. image.lockMovementX');

            test[widget]('destroy');

            QUnit.strictEqual(test.data('de_epagesExternalcontentUiCropslidedialog'), void(0), 'Instance destroyed. test.data(\'de_epagesExternalcontentUiCropslidedialog\')');

            // 2nd case.
            // Height Too Small.
            test = $('<div></div>').externalcontentUiCropslidedialog({
                width: 800,
                height: 400,
                url: 'cropslidedialog-testimage.png'
            });

            // Wait for image to load.
            setTimeout(function () {

                QUnit.strictEqual(
                    test.externalcontentUiCropslidedialog('option', 'width'),
                    800,
                    "test.externalcontentUiCropslidedialog('option', 'width')"
                );

                QUnit.strictEqual(
                    test.externalcontentUiCropslidedialog('option', 'height'),
                    400,
                    "test.externalcontentUiCropslidedialog('option', 'height')"
                );

                canvas = test[widget]('getFabricCanvas');
                image = test[widget]('getFabricImage');

                QUnit.strictEqual(
                    canvas.width,
                    800,
                    'Width of inserted canvas. canvas.width'
                );

                QUnit.strictEqual(
                    canvas.height,
                    400,
                    'Height of inserted canvas. canvas.height'
                );

                QUnit.strictEqual(
                    canvas.item(0) === image,
                    true,
                    'Image inserted into canvas. canvas.item(0) === image'
                );

                QUnit.strictEqual(
                    image.lockMovementY,
                    true,
                    'Image weidth too big. image.lockMovementY'
                );

                test[widget]('destroy');

                QUnit.strictEqual(
                    test.data('de_epagesExternalcontentUiCropslidedialog'),
                    void(0),
                    'Instance destroyed. test.data(\'de_epagesExternalcontentUiCropslidedialog\')'
                );

                // 3rd case.
                // Width too big after scaling to height.
                test = $('<div></div>').externalcontentUiCropslidedialog({
                    width: 899,
                    height: 299,
                    url: 'cropslidedialog-testimage.png'
                });

                 // Wait for image to load.
                setTimeout(function () {

                    QUnit.strictEqual(
                        test.externalcontentUiCropslidedialog('option', 'width'),
                        899,
                        "test.externalcontentUiCropslidedialog('option', 'width')"
                    );

                    QUnit.strictEqual(
                        test.externalcontentUiCropslidedialog('option', 'height'),
                        299,
                        "test.externalcontentUiCropslidedialog('option', 'height')"
                    );

                    canvas = test[widget]('getFabricCanvas');
                    image = test[widget]('getFabricImage');

                    QUnit.strictEqual(
                        canvas.width,
                        899,
                        'Width of inserted canvas. canvas.width'
                    );

                    QUnit.strictEqual(
                        canvas.height,
                        299,
                        'Height of inserted canvas. canvas.height'
                    );

                    QUnit.strictEqual(
                        canvas.item(0) === image,
                        true,
                        'Image inserted into canvas. canvas.item(0) === image'
                    );

                    QUnit.strictEqual(
                        image.lockMovementY,
                        true,
                        'Image width too big. image.lockMovementY'
                    );

                    test[widget]('destroy');

                    QUnit.strictEqual(
                        test.data('de_epagesExternalcontentUiCropslidedialog'),
                        void(0),
                        'Instance destroyed. test.data(\'de_epagesExternalcontentUiCropslidedialog\')'
                    );

                    // 4th case.
                    // Width and height too small.
                    test = $('<div></div>').externalcontentUiCropslidedialog({
                        width: 1000,
                        height: 400,
                        url: 'cropslidedialog-testimage.png'
                    });

                    // Wait for image to load.
                    setTimeout(function () {

                        QUnit.strictEqual(
                            test.externalcontentUiCropslidedialog('option', 'width'),
                            1000,
                            "test.externalcontentUiCropslidedialog('option', 'width')"
                        );

                        QUnit.strictEqual(
                            test.externalcontentUiCropslidedialog('option', 'height'),
                            400,
                            "test.externalcontentUiCropslidedialog('option', 'height')"
                        );

                        canvas = test[widget]('getFabricCanvas');
                        image = test[widget]('getFabricImage');

                        QUnit.strictEqual(
                            canvas.width,
                            1000,
                            'Width of inserted canvas. canvas.width'
                        );

                        QUnit.strictEqual(
                            canvas.height,
                            400,
                            'Height of inserted canvas. canvas.height'
                        );

                        QUnit.strictEqual(
                            canvas.item(0) === image,
                            true,
                            'Image inserted into canvas. canvas.item(0) === image'
                        );

                        QUnit.strictEqual(
                            image.lockMovementY,
                            true,
                            'Image weidth and height too small. image.lockMovementY'
                        );

                        QUnit.strictEqual(
                            image.lockMovementX,
                            true,
                            'Image weidth and height too small. image.lockMovementX'
                        );

                        test[widget]('destroy');

                        QUnit.strictEqual(
                            test.data('de_epagesExternalcontentUiCropslidedialog'),
                            void(0),
                            'Instance destroyed. test.data(\'de_epagesExternalcontentUiCropslidedialog\')'
                        );

                        QUnit.start();

                    }, 250);
                }, 250);
            }, 250);
        }, 250);
    });
});