/*global define, window, google, options, initBingMapIsExecuted*/

define([
    'testsuite/qunit',
    'jquery',

    './maps',
    '$ready!'
], function (QUnit, $) {
    QUnit.module('de_epages/externalcontent/ui/maps', {
        setup: function () {
            // Google Maps API for testing
            window.google = {
                load: function (type, version, params) {
                    params.callback();
                },
                maps: {
                    LatLng: function () {
                        return {};
                    },
                    Marker: function () {
                        return {
                            setTitle: function () {},
                            setMap: function () {}
                        };
                    },
                    Map: function () {
                        return {};
                    },
                    InfoWindow: function () {
                        return {};
                    },
                    event: {
                        addListener: function () {
                            return {};
                        }
                    }
                }
            };

            // Bing API for testing
            window.initBingMapIsExecuted = false;
            window.VEMapStyle = [];
            window.VEMapMode = [];
            window.VEMap = function (id) {
                this.id = id;
                this.LoadMap = function (location, zoom, type, fixedView, mode) {
                    window.initBingMapIsExecuted = true;
                    return {};
                };
                this.HideDashboard = function () {
                    return {};
                };
                this.AddPushpin = function (location) {
                    return { SetDescription: function (title) {} };
                };
                this.Dispose = function () {
                    return {};
                };
                this.ShowInfoBox = function () {
                    return {};
                };
            };

            window.VELatLong = function (lat, lng) {
                return {};
            };

            window.options = {
                service   : 'google',
                controls  : true,
                fixedView : false,
                width     : 300,
                height    : 200,
                lat       : 50.9287910,
                lng       : 11.5846220,
                zoom      : 5,
                title     : ''
            };

            $.ajax = function (o) {
                return {done: function (_func) { _func(); } };
            };
        },
        teardown: function () {}
    });

    QUnit.test('Test maps methods', 2, function () {
        $.each(['_create', 'destroy'], function (i, fn) {
            QUnit.equal(
                $.isFunction($.ui.externalcontentUiMaps.prototype[fn]),
                true,
                fn + ' is a method of maps'
            );
        });
    });

    QUnit.test('Test google maps', 4, function () {

        // add Google Widget
        $("#mapscontainer1").externalcontentUiMaps(
            $.extend(options, {
                type     : 'google',
                title    : 'My Title',
                callback : $.noop
            })
        );

        var googleMap = $("#mapscontainer1").find(":first-child");

        QUnit.equal(
            googleMap.prop("tagName"),
            "DIV",
            "the plugin is appended to the correct div"
        );

        QUnit.equal(
            googleMap.css("width"),
            "300px",
            "the appended div has the correct width"
        );

        QUnit.equal(
            googleMap.css("height"),
            "200px",
            "the appended div has the correct height"
        );

        $("#mapscontainer1").externalcontentUiMaps("destroy");

        QUnit.equal(
            $("#mapscontainer1").children().length,
            0,
            "All Children of the target <<Div>> of the Google Widget have been disappended after destroying the Widget"
        );
    });

    QUnit.test('Test bing map', 5, function () {
        // add Bing Widget
        // To avoid a getElementById conflict tell the Widget that no Mozilla is used. The function _bingCallback() is forced to go directly into the else case.
        if ($.browser.mozilla) {
            $.browser.mozilla = false;
        }

        $("#mapscontainer2").externalcontentUiMaps({
            type     : 'bing',
            width    : 600,
            height   : 400,
            title    : 'My Title',
            callback : $.noop,
            controls : false
        });

        var bingMap = $("#mapscontainer2").find(":first-child");

        QUnit.equal(
            initBingMapIsExecuted,
            true,
            "the function initBingMap is executed"
        );

        QUnit.equal(
            bingMap.attr("id").substr(0, 4),
            "uid_",
            "the plugin is appended to the correct div with correct id"
        );

        QUnit.equal(
            bingMap.css("width"),
            "600px",
            "the appended div has the correct width"
        );

        QUnit.equal(
            bingMap.css("height"),
            "400px",
            "the appended div has the correct height"
        );

        $("#mapscontainer2").externalcontentUiMaps("destroy");

        QUnit.equal(
            $("#mapscontainer2").children().length,
            0,
            "All Children of the target <<Div>> of the Bing Widget have been disappended after destroying the Widget"
        );
    });

    QUnit.test('Test klicktel maps', 2, function () {
        // add Klicktel Widget
        $("#mapscontainer3").externalcontentUiMaps({
            type : 'klicktel'
        });

        QUnit.equal(
            $("#mapscontainer3").find(":first-child").prop("tagName"),
            "IFRAME",
            "the plugin is appended to the correct div"
        );

        $("#mapscontainer3").externalcontentUiMaps("destroy");

        QUnit.equal(
            $("#mapscontainer3").children().length,
            0,
            "All Children of the target <<Div>> of the Klicktel Widget have been disappended after destroying the Widget"
        );
    });
});