/*global define, window*/

define([
    'testsuite/qunit',
    'jquery',

    './rss',
    '$ready!'
], function (QUnit, $) {
    QUnit.module('de_epages/externalcontent/ui/rss');

    QUnit.test('Test rss methods', 4, function () {
        $.each(['_create', '_createFeed', '_onFeedLoad', 'destroy'], function (i, fn) {
            QUnit.equal(
                $.isFunction($.ui.externalcontentUiRss.prototype[fn]),
                true,
                fn + ' is a method of rss'
            );
        });
    });

    QUnit.test('Test rss', function () {
        // Overwrite *ajax* for testing.
        $.ajax = function () {
            return {
                done : function (_func) {
                    // Init *google.load* dummy for testing.
                    window.google = {
                        load : function (_module, _version, _params) {
                            // Init *google.feeds* dummy for testing.
                            window.google.feeds = {
                                Feed : function (_url) {
                                    this.url = _url;
                                    window[_url] = {};
                                    this.includeHistoricalEntries = function () {
                                        window[_url].includeHistoricalEntries = true;
                                    };
                                    this.setNumEntries = function (_number) {
                                        window[_url].numEntries = _number;
                                    };
                                    this.load = function (_callback) {
                                        window[_url].calledFeedLoad = true;
                                        _callback({feed : {}});
                                        // The feed object is passed to *jQuery.tmpl*,
                                        // so we do not have to test anything here.
                                    };
                                }
                            };
                            _params.callback && _params.callback();
                        }
                    };
                    _func();
                }
            };
        };

        // Setup of test options.
        var letters = ['A', 'B', 'C'],
            options = {
                A   : {
                    url : 'testA'
                },
                B : {
                    url : 'testB',
                    numberOfEntries : 100
                },
                C : {
                    url : 'testC',
                    numberOfEntries : -1
                }
            };

        // Create three *otherbuttons* gadgets.
        $.each(letters, function (index, value) {
            $('div#' + value).externalcontentUiRss(options[value]);
        });

        // Testing *_create*.
        $.each(letters, function (index, value) {
            var $gNode = $('div#' + value);

            QUnit.equal(
                $gNode.children().length,
                1,
                'The container div has been added to ' + value + '. $gNode.children().length:'
            );

            QUnit.equal(
                window[options[value].url].calledFeedLoad,
                true,
                value + ' created an instance of *google.feeds.Feed* and called its *load* method. window[' + options[value].url + '].calledFeedLoad:'
            );

            QUnit.equal(
                window[options[value].url].numEntries,
                options[value].numberOfEntries || -1,
                value + ' told google to *load* the right number of entries (-1 means all available entries were loaded). window[' + options[value].url + '].numEntries:'
            );
        });

        // Testing *destroy*.
        $.each(letters, function (index, value) {
            var $gNode = $('div#' + value);

            $('div#' + value).externalcontentUiRss('destroy');

            QUnit.equal(
                $gNode.children().length,
                0,
                'The container div has been removed from ' + value + '. $gNode.children().length:'
            );

            QUnit.equal(
                $gNode.attr('style'),
                undefined,
                'Any previously added styles have been removed from ' + value + '. $gNode.attr("style"):'
            );
        });
    });
});