/**
 * @class ep
 * 
 */

/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.sprite
 *
 * @revision		$Revision: 1.5.2.1 $
 */

define("ep/sprite", [
	"ep",

	"ep/fn/sprite"
], function (ep) {

	ep.extend({
		/**
		 * Create a sprite icon.
		 * 
		 * The `ep.sprite()` method creates a span element an add css classes to style the icon.
		 * 
		 * ### Examples
		 * Create a arrow top-bottom icon is 16px size.
		 * 
		 * JavaScript:
		 * 
		 *     ep.sprite( 'arrow-tb', 's' );
		 * 
		 * Results:
		 * 
		 *     <span class="ep-sprite ep-sprite-s ep-sprite-arrow-tb"></span>
		 * 
		 * 
		 * ### Dependencies
		 * 
		 *  + `ep`
		 *  + `ep.fn.sprite`
		 * 
		 * @param {String} name A sprite icon name.
		 * @param {String} [size] Size of the icon (xs, s, l, m, l, xl, 64, 128), Default: m.
		 * 
		 * @method sprite
		 * @static 
		 * @member ep
		 * 
		 * @since 6.11.0
		 */
		sprite: function( name, size ){
			return ep('<span>').addSprite( name, size );
		}
	});

	return ep;

});