package DE_EPAGES::Object::t::Class;
use base DE_EPAGES::Database::API::Test;
use strict;
use Test::More tests => 35;
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias ExistsClassByAlias LoadObject );

my $DEFAULT_PACKAGE = 'DE_EPAGES::Object::API::Object::Object';

sub test {

    my $ClassObject = LoadClassByAlias( 'Object' );

    my $Class1 = $ClassObject->insertSubClass( { Alias => 'C1' } );
    ok( $Class1->id, 'insertSubClass 1' );

    my $Class2 = $Class1->insertSubClass( { Alias => 'C2' } );
    ok( $Class2->id, 'insertSubClass 2' );

    my $Class3 = $Class1->insertSubClass( { Alias => 'C3' } );
    ok( $Class3->id, 'insertSubClass 3' );

    my $Class4 = $Class3->insertSubClass( { Alias => 'C4' } );
    ok( $Class4->id, 'insertSubClass 4' );

    ok( ExistsClassByAlias( 'C3' ), 'existsName' );
    is( $Class3->id, LoadClassByAlias( 'C3' )->id, 'loadByName' );
    is( $Class3->alias, 'C3', 'name' );
    is( $Class3->alias, LoadObject( $Class3->id )->alias, 'load' );
    is( $Class3->get('Package'), $DEFAULT_PACKAGE, 'package' );
    is( $Class3->baseClass->alias, 'C1', 'baseClass' );

    is( scalar @{$Class3->baseClasses}, 4, 'baseClasses count' );
    is( $Class3->baseClasses->[-1]->alias, 'C3', 'baseClasses last' );
    is( $Class3->baseClasses->[0]->alias, 'BaseObject', 'baseClasses first' );
    is( $Class3->baseClasses->[1]->alias, 'Object', 'baseClasses second' );
    is( $Class3->baseClasses->[2]->alias, 'C1', 'baseClasses third' );

    is( scalar @{$Class1->subClasses}, 3, 'subClasses count' );
    is( scalar @{$Class1->directSubClasses}, 2, 'directSubClasses count' );
    is( $Class3->subClasses->[0]->alias, 'C4', 'subClasses->[0]->alias' );

    # attributes
    ok( !$Class1->existsAttribute( 'C1A1' ), 'existsAttribute' );
    $Class1->insertAttribute( {
        Alias => 'C1A1',
        Type => 'String',
        Length => 100,
        Package => 'DE_EPAGES::Object::API::Attributes::DefaultAttribute',
    } );
    ok( $Class1->existsAttribute( 'C1A1' ), 'insertAttribute' );
    is( $Class1->attribute( 'C1A1' )->alias, 'C1A1', 'attribute name' );
    is( $Class1->attribute( 'C1A1' )->get('Type'), 'String', 'attribute type' );
    is( $Class1->attribute( 'C1A1' )->get('Length'), 100, 'attribute length' );
    is( $Class1->attribute( 'C1A1' )->get('Package'), 'DE_EPAGES::Object::API::Attributes::DefaultAttribute', 'attribute package' );

    $Class3->insertAttribute( {
        Alias => 'C3A1',
        Type => 'Money',
        Package => 'DE_EPAGES::Object::API::Attributes::DefaultAttribute',
    } );
    ok( $Class3->existsAttribute( 'C3A1' ), 'insertAttribute 2' );
    ok( $Class4->existsAttribute( 'C1A1' ), 'exists inherited attribute 1' );
    ok( $Class4->existsAttribute( 'C3A1' ), 'exists inherited attribute 2' );
    is( $Class4->attribute( 'C3A1' )->get('Type'), 'Money', 'inherited attribute type' );
    is( $Class4->attribute( 'Alias' )->get('Type'), 'String', 'base attribute type 1' );
    is( $Class4->attribute( 'Class' )->get('Type'), 'Class', 'base attribute type 2' );
    is( $Class4->attribute( 'Parent' )->get('Type'), 'Object', 'base attribute type 3' );
    is( $Class4->attribute( 'Inherit' )->get('Type'), 'Boolean', 'base attribute type 4' );

    ok( 1 == scalar @{$Class1->directAttributes}, 'directAttributes count' );

    $Class1->deleteAttribute( 'C1A1' );
    ok( 0 == scalar @{$Class1->directAttributes}, 'deleteAttribute' );

    $Class3->delete;
    ok( 1 == scalar @{$Class1->subClasses}, 'subClasses count' );
}

sub cleanup {

    foreach my $ClassName ( qw( C1 C2 C3 C4 ) ) {
        next unless ExistsClassByAlias( $ClassName );
        LoadClassByAlias( $ClassName )->delete;
    }
}

__PACKAGE__->runScript;
