use strict;

use Readonly;
use Test::More;
use XML::Simple;
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Database::API::Connection qw (GetTestSite RunOnStore);
use DE_EPAGES::POSTPAY::t::API::TestHelper qw (Cleanup Init LoadPOSTPAYPayment LoadTestShop);
use DE_EPAGES::POSTPAY::API::Payment qw (SendPOSTPAYRequest);
use DE_EPAGES::POSTPAY::API::XML qw (CreatePOSTPAYXML);

Readonly my $BADREQUEST => 400;

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            # startUp
            ok( Init(), 'initialization' );
            my $Shop = LoadTestShop();
            my $PaymentMethod = LoadPOSTPAYPayment();
            my $hXMLRequestParams;
            # Create POSTPAY XML without content
            my $XML = CreatePOSTPAYXML($hXMLRequestParams,$Shop);
            my $Response = SendPOSTPAYRequest($XML,$PaymentMethod);
            my $Content = $Response->content;
            my $hResult = XMLin(
                $Content,
                KeepRoot => 1,
               'NoAttr'=>1,
            );
            is($hResult->{'error'}->{'error-code'},$BADREQUEST,'response from server');
            Cleanup();
            done_testing();
        }
    );
}

RunScript( Sub => \&Main );

1;