use strict;

use Test::More;
use XML::Simple;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::POSTPAY::t::API::TestHelper qw ( LoadTestShop Cleanup Init GetPOSTPAYTestXMLDirectory);
use DE_EPAGES::POSTPAY::t::API::Format qw (heading_1);
use DE_EPAGES::POSTPAY::API::Table::POSTPAYTransactions qw (ExistsPOSTPAYTransaction);
use DE_EPAGES::POSTPAY::API::XML qw (CreatePOSTPAYXML);
use DE_EPAGES::POSTPAY::API::Constants qw (VERSION XMLNSCHECKOUT XMLNSCOMMON );
use DE_EPAGES::Core::API::PerlTools qw (isValid);

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            #check for anononymus user aswell as for registered
            foreach (qw (LoadPOSTPAYLineItemPaymentAnonymusUser LoadPOSTPAYLineItemPaymentRegisteredUser)){
                # startUp
                heading_1($_);
                ok( Init(), 'initialization ');
                my $LineItemPayment = DE_EPAGES::POSTPAY::t::API::TestHelper->$_;
                # Test for correct Values
                my $hInfo;
                my $Shop = LoadTestShop();
                ##no critic qw(ProtectPrivateSubs)
                my $hXMLRequestParams   = DE_EPAGES::POSTPAY::API::Payment::_GetPOSTPAYSendParams(
                                                                                $LineItemPayment,
                                                                                $hInfo,
                                                                                $LineItemPayment->get('TransID'));
                ##use critic
                my $XML                 = CreatePOSTPAYXML($hXMLRequestParams,$Shop);

                my $hResult = XMLin(
                    $XML,
                    KeepRoot => 0,
                    'NoAttr'=>1,
                );
                my $Directory  = GetPOSTPAYTestXMLDirectory;
                my $hCorrectResult = XMLin(
                    $Directory.'SendBasket.xml',
                    KeepRoot => 0,
                    'NoAttr'=>1,
                );
                _checkHeaderStructure($hResult,$hCorrectResult);
                _checkAttributes($XML);
                _checkResult($hResult,$hCorrectResult);
                _checkURL($LineItemPayment);
                _checkTables($LineItemPayment);
            }
            Cleanup();
            done_testing();
        }
    );
}

sub _checkAttributes {
    my ($XML) = @_;
    my $Version = VERSION;
    my $XmlnsCommon = XMLNSCOMMON;
    my $XmlnsCheckout = XMLNSCHECKOUT;

    like(   $XML,
            qr/<\s*submitCartRequest.*xmlns=\"\Q$XmlnsCheckout\E".*>/x,
            'Attribute xmlns of submitCartRequest element correct');
    like(   $XML,
            qr/<\s*submitCartRequest.*version=\"\Q$Version\E\".*>/x,
            'Attribute Version of submitCartRequest element correct');
    like(   $XML,
            qr/<\s*submitCartRequest.*xmlns\:common=\"\Q$XmlnsCommon\E".*>/x,
            'Attribute xmlns:common of submitCartRequest element correct');
    like(   $XML,
            qr/<\s*header.*xmlns=\"\Q$XmlnsCommon\E".*>/x,
            'Attribute xmln of header element correct');
    return;
}

sub _checkHeaderStructure {
    my ($hResult,$hCorrectResult) = @_;
    is ($hResult->{'header'}->{'login'},$hCorrectResult->{'header'}->{'login'},'header login ok');
    # Just check if password is set, dont need clear text pw of test account on every epages isntallation
    ok (isValid($hResult->{'header'}->{'password'}),'header pwassword set');
    # Just check if language is set because it can differ with the system the test is run on
    ok (isValid($hResult->{'header'}->{'language'}),'header language set');
}

sub _checkURL {
    my ($LineItemPayment) = @_;
    my $Servlet;
    my $url = $LineItemPayment->checkoutRequest($Servlet);
    my $TransactionID =  $LineItemPayment->get('TransID');
    like($url, qr/http*\:\/\//,'return value is url');
    like($url, qr/$TransactionID/x,'return url matches basket transaction id');
}

sub _checkTables {
    my ($LineItemPayment) = @_;
    my $TransactionID =  $LineItemPayment->get('TransID');
    ok (ExistsPOSTPAYTransaction($TransactionID),'transaction is in postpaytransaction table');
}

sub _checkResult {
    my ($hResult,$hCorrectResult) = @_;
    is_deeply($hResult->{'shoppingCart'},$hCorrectResult->{'shoppingCart'},'XML is correct');
}

RunScript( Sub => \&Main );

1;