use strict;

use Test::More;
use XML::Simple;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::POSTPAY::t::API::TestHelper qw (LoadTestShop Cleanup Init GetPOSTPAYTestXMLDirectory);
use DE_EPAGES::POSTPAY::t::API::Format qw (heading_1);
use DE_EPAGES::POSTPAY::API::XML qw (CreatePOSTPAYXML);
use DE_EPAGES::POSTPAY::API::Constants qw (VERSION XMLNSCOMMON XMLNORDERS);
use DE_EPAGES::Core::API::PerlTools qw (isValid);

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            # check for anononymus user aswell as for registered
            foreach (qw (LoadPOSTPAYLineItemPaymentAnonymusUser LoadPOSTPAYLineItemPaymentRegisteredUser)){
                # check anonymus user
                # startUp
                heading_1($_);
                ok( Init(), 'initialization ');
                my $LineItemPayment = DE_EPAGES::POSTPAY::t::API::TestHelper->$_;
                my $Shop = LoadTestShop();
                ##no critic qw(ProtectPrivateSubs)
                my $hXMLRequestParams   = DE_EPAGES::POSTPAY::API::Payment::_GetRequestDetailsParams($LineItemPayment);
                ##use critic
                my $OrderDetailsXML     = CreatePOSTPAYXML($hXMLRequestParams,$Shop);

                my $hResult = XMLin(
                    $OrderDetailsXML,
                    KeepRoot => 0,
                    'NoAttr'=>1,
                );
                my $Directory  = GetPOSTPAYTestXMLDirectory;
                my $hCorrectResult = XMLin(
                    $Directory.'SendExternalID.xml',
                    KeepRoot => 0,
                    'NoAttr'=>1,
                );
                _checkHeaderStructure($hResult,$hCorrectResult);
                _checkAttributes($OrderDetailsXML);
                _checkResult($hResult,$hCorrectResult);
            }
            Cleanup();
            done_testing();
        }
    );
}
sub _checkHeaderStructure {
    my ($hResult,$hCorrectResult) = @_;
    is ($hResult->{'header'}->{'login'},$hCorrectResult->{'header'}->{'login'},'header login ok');
    # Just check if password is set, dont need clear text pw of test account on every epages isntallation
    ok (isValid($hResult->{'header'}->{'password'}),'header password set');
    # Just check if language is set because it can differ with the system the test is run on
    ok (isValid($hResult->{'header'}->{'language'}),'header language set');
}

sub _checkAttributes {
    my ($XML) = @_;
    my $Version = VERSION;
    my $XmlnsOrders = XMLNORDERS;
    my $XmlnsCommon = XMLNSCOMMON;

    like($XML, qr/<\s*queryRequest.*xmlns=\"\Q$XmlnsOrders\E".*>/x,'Attribute xmlns of queryRequest element correct');
    like($XML, qr/<\s*queryRequest.*version=\"\Q$Version\E\".*>/x,'Attribute Version of queryRequest element correct');
    like($XML, qr/<\s*queryRequest.*xmlns\:common=\"\Q$XmlnsCommon\E".*>/x,'Attribute xmlns:common of queryRequest element correct');
    like($XML, qr/<\s*header.*xmlns=\"\Q$XmlnsCommon\E".*>/x,'Attribute xmln of header element correct');
    return;
}

sub _checkResult {
    my ($hResult,$hCorrectResult) = @_;
    is_deeply($hResult->{'orders'},$hCorrectResult->{'orders'},'XML is correct');
}

RunScript( Sub => \&Main );

1;