use strict;

use Test::More;
use XML::Simple;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::POSTPAY::t::API::TestHelper qw (Cleanup Init GetPOSTPAYTestXMLDirectory ExistsPOSTPAYCustomer LoadPOSTPAYCustomer );
use DE_EPAGES::POSTPAY::t::API::Format qw (heading_1 heading_2);
use DE_EPAGES::POSTPAY::API::Payment qw (HandleQueryRequestResponse);
use DE_EPAGES::Object::API::Table::Country qw (GetInfoCountryByCode2);
use DE_EPAGES::Core::API::File qw (GetFileContent);

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            # check for anononymus user aswell as for registered
            foreach (qw (LoadPOSTPAYLineItemPaymentAnonymusUser LoadPOSTPAYLineItemPaymentRegisteredUser)){
                # check anonymus user
                # startUp
                heading_1($_);
                ok( Init(), 'initialization ');
                my $LineItemPayment = DE_EPAGES::POSTPAY::t::API::TestHelper->$_;

                # Case 1: Same Address (Shipping and Billing Address is the same)
                my $Directory  = GetPOSTPAYTestXMLDirectory;
                my $XML = ${GetFileContent($Directory.'ResponseExternalID.xml')};
                my $hResult = XMLin(
                    $XML,
                    KeepRoot => 1,
                    'NoAttr'=>1,
                );
                ##no critic qw(RequireExistsError)
                eval {
                    HandleQueryRequestResponse($hResult->{'queryResponse'}->{'order'},$LineItemPayment);
                };
                ##use critic
                heading_2('checkCustomerWithSameAddress');
                _checkCustomerWithSameAddress($hResult);
                _checkOrder($LineItemPayment,$hResult);

                # Case 2: Different Address
                my $XML2 = ${GetFileContent($Directory.'ResponseExternalID2.xml')};
                $hResult = XMLin(
                    $XML2,
                    KeepRoot => 1,
                    'NoAttr'=>1,
                );
                ##no critic qw(RequireExistsError)
                eval {
                    HandleQueryRequestResponse($hResult->{'queryResponse'}->{'order'},$LineItemPayment);
                };
                ##use critic
                heading_2('checkCustomerWithDifferentAddress');
                _checkCustomerWithDifferentAddress($LineItemPayment,$hResult);
                _checkOrder($LineItemPayment,$hResult);

            }
            Cleanup();
            done_testing();
        }
    );
}

sub _checkCustomerWithSameAddress {
    my ($hResult) = @_;

    ok(ExistsPOSTPAYCustomer(),'Customer created');
    return unless ExistsPOSTPAYCustomer;
    my $Customer = LoadPOSTPAYCustomer();

    # created Address
    my $BillingAddress = $Customer->get('BillingAddress');
    # Address from xml
    my $hBillingCorrectAddress = $hResult->{'queryResponse'}->{'order'}->{'billingAddress'};
    _CompareAddress($BillingAddress,$hBillingCorrectAddress);

    # created Address
    my $ShippingAddress = $Customer->get('BillingAddress');
    # Address from xml
    my $hDeliveryAddress = $hResult->{'queryResponse'}->{'order'}->{'deliveryAddress'};
    #compare POSTPAY Addresses in xml with addresses saved in shop
    _CompareAddress($ShippingAddress,$hDeliveryAddress);
}
sub _checkCustomerWithDifferentAddress {
    my ($LineItemPayment,$hResult) = @_;

    ok(ExistsPOSTPAYCustomer(),'Customer created');
    return unless ExistsPOSTPAYCustomer;

    my $Customer = LoadPOSTPAYCustomer();
    my $LineItemContainer = $LineItemPayment->parent;
    my $Basket = $LineItemContainer->parent;

    # created Address
    my $BillingAddress = $Customer->get('BillingAddress');
    # Address from xml
    my $hBillingCorrectAddress = $hResult->{'queryResponse'}->{'order'}->{'billingAddress'};
    _CompareAddress($BillingAddress,$hBillingCorrectAddress);
    # created Address

    my $ShippingAddress = $Basket->get('ShippingAddress');
    # Address from xml
    my $hDeliveryAddress = $hResult->{'queryResponse'}->{'order'}->{'deliveryAddress'};
    #compare POSTPAY Addresses in xml with addresses saved in shop
    _CompareAddress($ShippingAddress,$hDeliveryAddress);
}
sub _CompareAddress{
    my ($Address,$hCorrectAddress) = @_;
    ok(defined($Address),'Address defined');
    return unless defined($Address);

    my $AddressAlias= $Address->alias;
    is( $Address->get('Salutation'),
        $hCorrectAddress->{'salutation'},
        $AddressAlias.': Salutation correct');
    is( $Address->get('FirstName'),
        $hCorrectAddress->{'firstName'},
        $AddressAlias.': FirstName correct');
    is( $Address->get('LastName'),
        $hCorrectAddress->{'lastName'},
        $AddressAlias.': LastName correct');
    is( $Address->get('City'),
        $hCorrectAddress->{'city'},
        $AddressAlias.': City correct');
    is( $Address->get('Street2'),
        $hCorrectAddress->{'addressAddition'},
        $AddressAlias.': Street2 correct');
    is( $Address->get('Title'),
        $hCorrectAddress->{'title'},
        $AddressAlias.': Title correct');
    is( $Address->get('CountryID'),
        GetInfoCountryByCode2(uc($hCorrectAddress->{'country'}))->{'CountryID'},
        $AddressAlias.': CountryID correct');
    is( $Address->get('Zipcode'),
        $hCorrectAddress->{'zipCode'},
        $AddressAlias.': Zipcode correct');
    is( $Address->get('Street'),
        $hCorrectAddress->{'street'}.' '.$hCorrectAddress->{'houseNumber'},
        $AddressAlias.': Street correct');
    return;
}

sub _checkOrder {
    my ($LineItemPayment,$hResult) = @_;
    my $LineItemContainer = $LineItemPayment->parent;
    my $Basket = $LineItemContainer->parent;

    ok(defined ($Basket),'Basket defined');
    return unless defined ($Basket);
    ok(defined ($Basket->get('Customer')),'Customer set on Basket');
    return unless defined $Basket->get('Customer');

    is( $Basket->get('Customer')->alias,
        LoadPOSTPAYCustomer()->alias,
        'Customer set on Basket is correct');
    is( $LineItemPayment->get('TransID2'),
        $hResult->{'queryResponse'}->{'order'}->{'orderId'},
        'OrderId from POSTPAY is set on Basket');
    is( $LineItemContainer->get('GrandTotal'),
        $hResult->{'queryResponse'}->{'order'}->{'totalPrice'},
        'Total Amount is correct');
    return;
}

RunScript( Sub => \&Main );

1;