## no critic qw( ProhibitMagicNumbers RequireCommentForExportedFunctions )
package DE_EPAGES::ShopTransfer::t::DBIMySQL::t;
use base DE_EPAGES::Database::API::Test;

use strict;
use Test::More tests => 12;
use DE_EPAGES::ShopTransfer::API::DBImySQL;

sub testReserveObjectIDs {
    my $self = shift;
    my $dbi = $self->dbi;

    my $DBI = DE_EPAGES::ShopTransfer::API::DBImySQL->new;
    my $MaxID = $dbi->execute('SELECT MAX(objectid) FROM object')->[0]->[0];
    my $AutoIncrement = $DBI->autoIncrementNextID('object');
    is( $AutoIncrement, 1 + $MaxID, 'initial auto_increment' );

    my $NextID = $DBI->reserveIdentifier( 'object', 10 );
    is( $NextID, $AutoIncrement, 'reserveIdentifier' );

    $NextID = $DBI->reserveIdentifier( 'object', 10 );
    is( $NextID, 10 + $AutoIncrement, 'reserveIdentifier 2nd' );

    $NextID = $DBI->reserveIdentifier( 'object', 0 );
    is( $NextID, 20 + $AutoIncrement, 'reserveIdentifier(0)' );

    $NextID = $DBI->reserveIdentifier( 'object', 0 );
    is( $NextID, 20 + $AutoIncrement, 'reserveIdentifier(0) 2nd' );

    $dbi->do(<<'END_SQL');
INSERT INTO object(classid, alias, position, guid, inherit, creationdate, siteid)
VALUES(0, 'testReserveIdentifier', 0, 'testReserveIdentifier', 0, NOW(), 1)
END_SQL

    my $LastID = $dbi->dbh->{'mysql_insertid'};
    is( $LastID, 20 + $AutoIncrement, 'insert object' );
    $dbi->do('DELETE FROM object WHERE guid = \'testReserveIdentifier\'');
}

sub testReserveLanguageIDs {
    my $self = shift;
    my $dbi = $self->dbi;

    my $DBI = DE_EPAGES::ShopTransfer::API::DBImySQL->new;
    my $MaxID = $dbi->execute('SELECT MAX(languageid) FROM language')->[0]->[0];
    my $AutoIncrement = $DBI->autoIncrementNextID('language');
    is( $AutoIncrement, 1 + $MaxID, 'initial auto_increment' );

    my $NextID = $DBI->reserveIdentifier( 'language', 10 );
    is( $NextID, $AutoIncrement, 'reserveIdentifier' );

    $NextID = $DBI->reserveIdentifier( 'language', 10 );
    is( $NextID, 10 + $AutoIncrement, 'reserveIdentifier 2nd' );

    $NextID = $DBI->reserveIdentifier( 'language', 0 );
    is( $NextID, 20 + $AutoIncrement, 'reserveIdentifier(0)' );

    $NextID = $DBI->reserveIdentifier( 'language', 0 );
    is( $NextID, 20 + $AutoIncrement, 'reserveIdentifier(0) 2nd' );

    $dbi->do('INSERT INTO language(code3, isused, isforbackoffice) VALUES (\'XXX\', 0, 0)');
    my $LastID = $dbi->dbh->{'mysql_insertid'};

    is( $LastID, 20 + $AutoIncrement, 'insert language' );
    $dbi->do('DELETE FROM object WHERE guid = \'testReserveIdentifier\'');
}

sub test {
    my $self = shift;
    my $dbi = $self->dbi;

    SKIP: {
        skip( 'this test is only for mysql', 12 ) if $dbi->driverName ne 'mysql';
        $self->testReserveObjectIDs;
        $self->testReserveLanguageIDs;
    }
}

sub cleanup {
    my $self = shift;
    my $dbi = $self->dbi;

    $dbi->do('DELETE FROM object WHERE guid = \'testReserveIdentifier\'');
    my $NextObjectID = 1 + $dbi->execute('SELECT MAX(objectid) FROM object')->[0]->[0];
    $dbi->do("ALTER TABLE object AUTO_INCREMENT = $NextObjectID");

    $dbi->do('DELETE FROM language WHERE code3 = \'XXX\'');
    my $NextLanguageID = 1 + $dbi->execute('SELECT MAX(languageid) FROM language')->[0]->[0];
    $dbi->do("ALTER TABLE language AUTO_INCREMENT = $NextLanguageID");
}

__PACKAGE__->runScript;
